require 'yaml'

module Ludwig
  class CLI
    include Commander::Methods

    def run
      program :version, Ludwig::VERSION
      program :description, 'Ludwig, the famous composer'

      @config_file = 'ludwig.yml'

      global_option('-d', '--debug', 'Enable debug mode') { $debug = true }
      global_option('-c', '--config filename', String, 'Set config filename') { |config_file| @config_file = config_file }

      command :compose do |c|
        c.syntax = 'ludwig compose [options]'
        c.summary = 'Generate docker-compose.yml'
        c.action do |args, options|
          generate_compose_file
        end
      end

      command :up do |c|
        c.syntax = 'ludwig up [options]'
        c.summary = 'Generate docker-compose.yml and launch docker-compose'
        c.action do |args, options|
          generate_compose_file
          system 'docker-compose up'
        end
      end

      run!
    end

    private

    def config
      @config ||= YAML.load_file @config_file
    end

    def generate_compose_file
      Ludwig::Composer.new(config).write_yaml
    end
  end
end
