module Ludwig
  Service = Struct.new(:project, :name, :data, :config) do
    def generate_yaml
      transform_data
      merge_extra_links
      new_data
    end

    private

    def merge_extra_links
      if extra_links = config["extra_links"][name]
        new_data["links"] ||= []
        new_data["links"] += extra_links.map do |app|
          "#{sanitize(app)}_#{config["default_app"]}:#{sanitize(app)}"
        end
      end
    end

    def new_data
      @new_data ||= {}
    end

    def sanitize(input)
      input.gsub(/-/, "_")
    end

    def transform_data
      data.each do |key, value|
        new_data[key] = try("transform_#{key}", value) || value
      end
    end

    def transform_links(links)
      links.map do |link|
        link_name, link_alias = link.split(":")
        link_alias = link_name if link_alias.nil?
        "#{sanitize(project)}_#{link_name}:#{link_alias}"
      end
    end

    def transform_path(path)
      "../#{project}/#{path}"
    end
    alias_method :transform_build,    :transform_path
    alias_method :transform_env_file, :transform_path

    def transform_ports(ports)
      return ports if config['expose_ports'][name]
      ports.map { |port| port.to_s.split(':').last }
    end

    def transform_volumes(volumes)
      volumes.map do |volume|
        local, remote = volume.split(":")
        local =~ /^\.\/?/ ? "#{transform_path(local)}:#{remote}" : volume
      end
    end

    def try(*a, &b)
      respond_to?(a.first, true) ? send(*a, &b) : nil
    end
  end
end
